/**
 * Created by zhang on 2018/9/11.
 */
import BaseController from "../../common/base/base_controller";
let config = require("../../../config/config.js");
let logger = require('../../../logs/log_helper').logger;
import calcProgramFacade from "../facade/calc_program_facade";
import CompilationModel from '../../users/models/compilation_model';

class CalcProgramController extends BaseController {
    async main(request, response) {
        let compilationModel = new CompilationModel();
        let compilationList = await compilationModel.getCompilationList({_id: 1, name: 1});
        compilationList.unshift({_id: 'all', name: '所有'});
        let activeCompilation = compilationList.find(compilation => compilation._id.toString() === request.query.filter);
        if (activeCompilation) {
            activeCompilation.active = 'active';
        } else {
            compilationList[0].active = 'active'
        }
        let filter = request.query.filter ? {compilationId: request.query.filter} : {};
        let calcProgramLibs = await calcProgramFacade.findByCondition(filter, {templates: 0}, false);
        let randerData = {
            title: '计算程序模板库',
            userAccount: request.session.managerData.username,
            userID: request.session.managerData.userID,
            calcProgramLibs: calcProgramLibs,
            compilationList: compilationList,
            layout: 'maintain/common/html/layout'
        };
        response.render("maintain/calc_program_lib/html/main", randerData);
    }
    async addLib(request, response){
        try {
            await calcProgramFacade.addLib(request.body);
        }catch (error) {
            console.log(error);
        }
        response.redirect(request.headers.referer);
    }
    async findLib(request, response){
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let conditions={ID:data.ID};
            let resultData = await calcProgramFacade.findByCondition(conditions);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async saveLib(request, response){
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData= await calcProgramFacade.saveLib(data);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async deleteLibByID(request,response){
        logger.info(`delete calcProgramLib ${request.ip}`);
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData= await calcProgramFacade.deleteLibByID(data.ID);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async edit(request,response){
        //先取出替换库信息：
        let libID = request.params.libID;
        let programLib = await calcProgramFacade.findByCondition({'ID':libID});
        if(programLib){
            let randerData = {
                title:'计算程序模板库',
                mainURL:'/calcProgram/main',
                libName:programLib.libName,
                userAccount: request.session.managerData.username,
                userID: request.session.managerData.userID,
                templateList:JSON.stringify(programLib.templates),
                libID:libID,
                LicenseKey:config.getLicenseKey(process.env.NODE_ENV),
                layout: 'maintain/common/html/edit_layout'
            };
            response.render("maintain/calc_program_lib/html/edit", randerData);
        }else {
            response.redirect(request.headers.referer);
        }
    }
}

export default CalcProgramController;