/**
 * Created by zhang on 2018/9/10.
 */
import BaseController from "../../common/base/base_controller";
let config = require("../../../config/config.js");
let logger = require('../../../logs/log_helper').logger;
import feeRateFacade from "../facade/fee_rate_facade";
import CompilationModel from '../../users/models/compilation_model';

class FeeRateController extends BaseController{
    async main(request, response) {
        let compilationModel = new CompilationModel();
        let compilationList = await compilationModel.getCompilationList({_id: 1, name: 1});
        compilationList.unshift({_id: 'all', name: '所有'});
        let activeCompilation = compilationList.find(compilation => compilation._id.toString() === request.query.filter);
        if (activeCompilation) {
            activeCompilation.active = 'active';
        } else {
            compilationList[0].active = 'active'
        }
        let filter = request.query.filter ? {compilationId: request.query.filter} : {};
        let feeRateLibs = await feeRateFacade.findByCondition(filter,{rates:0},false);
        let randerData = {
            title:'费率标准库',
            userAccount: request.session.managerData.username,
            userID: request.session.managerData.userID,
            feeRateLibs:feeRateLibs,
            compilationList: compilationList,
            layout: 'maintain/common/html/layout'
        };
        response.render("maintain/fee_rate_lib/html/main", randerData);
    }
    async addLib(request, response){
        try {
            await feeRateFacade.addLib(request.body);
        }catch (error) {
            console.log(error);
        }
        response.redirect(request.headers.referer);
    }
    async findLib(request, response){
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let conditions={ID:data.ID};
            let resultData = await feeRateFacade.findByCondition(conditions);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async saveLib(request, response){
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData= await feeRateFacade.saveLib(data);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async deleteLibByID(request,response){
        logger.info(`delete feeRateLib ${request.ip}`);
        let result={
            error:0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData= await feeRateFacade.deleteLibByID(data.ID);
            result.data=resultData;
        }catch (err){
            console.log(err);
            result.error=1;
            result.message = err.message;
        }
        response.json(result);
    }
    async edit(request,response){
        //先取出替换库信息：
        let libID = request.params.libID;
        let feeRateLib = await feeRateFacade.findByCondition({'ID':libID});
        if(feeRateLib){
            let randerData = {
                title:'费率标准库',
                mainURL:'/feeRate/main',
                libName:feeRateLib.libName,
                userAccount: request.session.managerData.username,
                userID: request.session.managerData.userID,
                rateList:JSON.stringify(feeRateLib.rates),
                libID:libID,
                LicenseKey:config.getLicenseKey(process.env.NODE_ENV),
                layout: 'maintain/common/html/edit_layout'
            };
            response.render("maintain/fee_rate_lib/html/edit", randerData);
        }else {
            response.redirect(request.headers.referer);
        }
    }
}

export default FeeRateController;